'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = resembleImage;

var _assetResolver = require('asset-resolver');

var _assetResolver2 = _interopRequireDefault(_assetResolver);

var _paper = require('paper');

var _paper2 = _interopRequireDefault(_paper);

var _imageSize = require('image-size');

var _imageSize2 = _interopRequireDefault(_imageSize);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function scaleValue(value, min, max) {
    var newMin = 0;
    var newMax = 100;
    var percent = (value - min) / (max - min);
    return percent * (newMax - newMin) + newMin;
}

function colourStopFactory(width) {
    return function colourStop(colour, index) {
        return {
            colour: colour,
            position: scaleValue(index, 0, width)
        };
    };
}

function resolveFidelity(width, pair) {
    if (!pair) {
        throw new Error('Expected a <number> or <percentage> value for fidelity.');
    }
    var number = parseFloat(pair.number);
    if (number === 0) {
        throw new Error('Expected a fidelity greater than 0.');
    }
    if (pair.unit === '%') {
        return width * (number / 100);
    }
    return number;
}

function resembleImage(image, _ref) {
    var generator = _ref.generator;
    var fidelity = _ref.fidelity;

    return _assetResolver2.default.getResource(image).then(function (data) {
        return data.contents;
    }).then(_imageSize2.default).then(function (_ref2) {
        var width = _ref2.width;
        var height = _ref2.height;

        return new Promise(function (resolve, reject) {
            _paper2.default.setup(new _paper2.default.Size(width, height));
            var raster = new _paper2.default.Raster(image);
            var colourStop = colourStopFactory(width);

            raster.onLoad = function () {
                var stops = [];
                raster.position = _paper2.default.view.center;
                var chunk = void 0;
                try {
                    chunk = resolveFidelity(width, fidelity);
                } catch (e) {
                    return reject(e);
                }

                for (var i = 0; i < width; i += chunk) {
                    var rect = new _paper2.default.Path.Rectangle(i, 0, chunk, height);
                    stops.push(colourStop(raster.getAverageColor(rect).toCSS(true), i));
                }

                return resolve(generator(stops));
            };
        });
    });
}